function [Xout,Sout]=e1g(Xin,tisk)
%% reseni Riemannova problemu Godunovovou metodou
% prepis programu uvedeneho v knize
% E.F.Toro: Riemann Solvers and Numerical Methods for Fluid Dynamics.
% A Practical Introduction. Springer-Verlag, Berlin, 2009
%
% vstupni data:
%
% Xin  ... vstupni data, strukura obsahujici
%
%   DOMLEN  ... delka intervalu
%   DIAPH1 ... poloha prvni prepazky
%   CELLS   ... pocet bunek
%   GAMMA   ... Poissonova adiabaticka konstanta
%   TIMEOU  ... doba reseni
%   DLINIT  ... pocatecni hustota vlevo
%   ULINIT  ... pocatecni rychlost vlevo
%   PLINIT  ... pocatecni tlak uprostred
%   DMINIT  ... pocatecni hustota uprostred
%   UMINIT  ... pocatecni rychlost uprostred
%   PMINIT  ... pocatecni tlak uprostred
%   DRINIT  ... pocatecni hustota vpravo
%   URINIT  ... pocatecni rychlost vpravo
%   PRINIT  ... pocatecni tlak vpravo
%   DIAPH2 ... poloha druhe prepazky
%   CFLCOE  ... CFL koeficient
%   IBCLEF  ... okrajova podminka vlavo,    = 0 ==> extrapolace
%   IBCRIG  ... okrajova podminka vpravo    = 1 ==> odraz
%   NFREQU  ... frekvence vystupu
%   NTMAXI  ... maximalni pocet casovych kroku
%   PSCALE  ... normalizacni konstanta
%   INTFLX  ... cislo solveru  = 1, Roe
%                              = 2, HLLC
%                              = 3, HLL
%                              = 4, Rusakov
%                              = 5, Vijayasundaram
%                              = 6, Steger-Warming
%                              = 7, van Leer (podle Feistauera)
%                              = 8, van Leer (podle Tora)
%
% tisk   ...  = true  ==> pomocne tisky solveru
%             = false ==> bez pomocnych tisku solveru
%
% vystupni udaje:
%
% Xout ... pole typu (5,CELLS), jejiz jednotlive radky obsahuji: 
%          
% Xout(1,:) ... souradnice stredu bunek
% Xout(2,:) ... hustoty ve stredech bunek
% Xout(3,:) ... rychlosti ve stredech bunek
% Xout(4,:) ... tlaky ve stredech bunek
% Xout(5,:) ... merne vnitrni energie ve stredech bunek
%% vypocet
[DOMLEN,DIAPH1,CELLS,GAMMA,TIMEOU,...
 DLINIT,ULINIT,PLINIT,DMINIT,UMINIT,PMINIT,DRINIT,URINIT,PRINIT,...
 DIAPH2,CFLCOE,IBCLEF,IBCRIG,NFREQU,NTMAXI,PSCALE,INTFLX]=Xin{1:end};

TIME=0; TIMTOL=1e-6;

BOUNDA={IBCLEF,IBCRIG};
DOMAIN={DOMLEN,DIAPH1,DIAPH2};
INISTA={DLINIT,ULINIT,PLINIT,DMINIT,UMINIT,PMINIT,DRINIT,URINIT,PRINIT};
met=char('Roe','HLLC','HLL','Rusakov',...
         'Vijayasundaram', 'Steger-Warming',...
         'Van Leer (Faistauer)','Van Leer (Toro)');
global method;
method=deblank(met(INTFLX,:));
[GAMMAS,PRIMIT,CONSER,MESHPA]=...  % pocatecni podminky
    initia(CELLS,GAMMA,DOMAIN,INISTA);
if tisk
  fprintf(['---------------------------------------------\n',... 
           'krok   delka kroku      cas       konecny cas\n',...
           '---------------------------------------------\n']);
end
for N=1:NTMAXI % cyklus pres cas
  [PRIMIT]=bcondi(CELLS,PRIMIT,BOUNDA); % okrajove podminky
  [SOUNDS,MESHPA,TIME]=cflcon(CFLCOE,CELLS,N,TIME,TIMEOU,...
                              GAMMAS,PRIMIT,MESHPA); % delka krok 
  [DT,~]=MESHPA{1:end};
  
  if INTFLX==1
    [FLUXES,CONSER]=roe(CELLS,PRIMIT,SOUNDS,CONSER,GAMMAS);
  elseif INTFLX==2
    [FLUXES,CONSER]=hllc(CELLS,PRIMIT,SOUNDS,CONSER,GAMMAS);      
  elseif INTFLX==3
    [FLUXES,CONSER]=hll(CELLS,PRIMIT,SOUNDS,CONSER,GAMMAS);      
  elseif INTFLX==4
    [FLUXES,CONSER]=rusanov(CELLS,PRIMIT,SOUNDS,CONSER,GAMMAS);      
  elseif INTFLX==5
    [FLUXES,CONSER]=vija(CELLS,PRIMIT,CONSER,GAMMAS);  
  elseif INTFLX==6
    [FLUXES,CONSER]=stewa(CELLS,PRIMIT,SOUNDS,CONSER,GAMMAS);      
  elseif INTFLX==7
    [FLUXES,CONSER]=vanleer1(CELLS,PRIMIT,SOUNDS,CONSER,GAMMAS);  
  elseif INTFLX==8
    [FLUXES,CONSER]=vanleer2(CELLS,PRIMIT,SOUNDS,CONSER,GAMMAS);  
  end
  
  [PRIMIT,CONSER]=update(CELLS,GAMMAS,PRIMIT,CONSER,FLUXES,MESHPA);
  if (mod(N,NFREQU)==0) && tisk
    fprintf('%4d  %10.6f   %10.6f     %10.6f\n',N,DT,TIME,TIMEOU);
  end;
  TIMDIF=abs(TIME-TIMEOU);
  if TIMDIF<=TIMTOL
    if tisk
      fprintf(['---------------------------------------------\n',...
               'pocet kroku = %d\n',...
               '---------------------------------------------\n'],N);
    end;
    break;
  end
end % cyklus pres cas
if N<NTMAXI,Sout=true; else Sout=false; end;
Xout=output(CELLS,PSCALE,GAMMAS,PRIMIT,MESHPA);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [GAMMAS,PRIMIT,CONSER,MESHPA]=initia(CELLS,GAMMA,DOMAIN,INISTA) 
%% nastaveni pocatecnich hodnot promennych
[DOMLEN,DIAPH1,DIAPH2]=DOMAIN{1:end};
[DLINIT,ULINIT,PLINIT,DMINIT,UMINIT,PMINIT,DRINIT,URINIT,PRINIT]=...
 INISTA{1:end};

IDIM=CELLS;
D=zeros(IDIM+4,1); % pseudodeklarace
U=zeros(IDIM+4,1);
P=zeros(IDIM+4,1);
CS=zeros(3,IDIM+4);                                                                     

G1=(GAMMA-1)/(2*GAMMA); % konstanty GAMMAS
G2=(GAMMA+1)/(2*GAMMA);
G3=2*GAMMA/(GAMMA-1);
G4=2/(GAMMA-1);
G5=2/(GAMMA+1);
G6=(GAMMA-1)/(GAMMA+1);
G7=(GAMMA-1)/2;
G8=GAMMA-1;

DX=DOMLEN/CELLS;
for II=1:CELLS % cyklus pres vsechny bunky
  XPOS=(II-0.5)*DX;
  I=II+2; % korekce indexu
% nastaveni pocatecnich hodnot fyzikalnich promennych
  if XPOS<=DIAPH1 % vlevo od prvni prepazky
    D(I)=DLINIT;
    U(I)=ULINIT;
    P(I)=PLINIT;
  elseif (XPOS>DIAPH1)&&(XPOS<=DIAPH2) % mezi prepazkama
    D(I)=DMINIT;
    U(I)=UMINIT;
    P(I)=PMINIT;
  else % napravo od druhe prepazky
    D(I)=DRINIT;
    U(I)=URINIT;
    P(I)=PRINIT;
  end
% nastaveni pocatecnich hodnot konzervativnich promennych  
  CS(1,I)=D(I);
  CS(2,I)=D(I)*U(I);
  CS(3,I)=0.5*CS(2,I)*U(I)+P(I)/G8;
end;

GAMMAS={GAMMA,G1,G2,G3,G4,G5,G6,G7,G8};
PRIMIT={D,U,P};
CONSER={CS};
MESHPA={0,DX};
function [PRIMIT]=bcondi(CELLS,PRIMIT,BOUNDA)
%% uplatneni okrajovych podminek
[IBCLEF,IBCRIG]=BOUNDA{1:end};
[D,U,P]=PRIMIT{1:end}; 

if IBCLEF==0 % extrapolace vlevo
  D(2)=D(3);
  U(2)=U(3);
  P(2)=P(3);
else % odraz vlevo
  D(2)=D(3);
  U(2)=-U(3);
  P(2)=P(3);
end
if IBCRIG==0 % extrapolace vpravo
  D(CELLS+3)=D(CELLS+2);
  U(CELLS+3)=U(CELLS+2);
  P(CELLS+3)=P(CELLS+2);
else % odraz vpravo
  D(CELLS+3)=D(CELLS+2);
  U(CELLS+3)=-U(CELLS+2);
  P(CELLS+3)=P(CELLS+2);
end

PRIMIT={D,U,P};

function [SOUNDS,MESHPA,TIME]=cflcon(CFLCOE,CELLS,N,TIME,TIMEOU,...
                                     GAMMAS,PRIMIT,MESHPA)
%%  urceni delky casoveho kroku
global method;
GAMMA=GAMMAS{1};
[D,U,P]=PRIMIT{1:end};
[~,DX]=MESHPA{1:end}; 

C=zeros(CELLS+4,1); % rychlost zvuku, pseudodeklarace
SMAX=0;
for II=0:CELLS+1 % urceni maximalni rychlosti SMAX
  I=II+2;
  CI2=GAMMA*P(I)/D(I);
  assert(~(CI2<0),['***CHYBA*** metoda : %s, ',...
                   'komplexn rychlost zvuku'],method);
  C(I)=sqrt(CI2);
  SBEXTD=abs(U(I))+C(I); % nejvetsi vlastni cislo bunky I
  if SBEXTD>SMAX, SMAX=SBEXTD; end;
end
DT=CFLCOE*DX/SMAX; % delka kroku
if N<=5, DT=0.2*DT; end; % startovaci kroky se zkrati
if TIME+DT>TIMEOU, DT=TIMEOU-TIME; end; % kraceni posledniho kroku
TIME=TIME+DT; % novy cas

MESHPA={DT,DX}; 
SOUNDS={C};
function [Xout]= output(CELLS,PSCALE,GAMMAS,PRIMIT,MESHPA)
%% vystup vysledku
[~,~,~,~,~,~,~,~,G8]=GAMMAS{1:end};
[D,U,P]=PRIMIT{1:end};
[~,DX]=MESHPA{1:end};

id=3:CELLS+2; 
Xout=[DX*(id-2.5)' D(id) U(id) P(id)/PSCALE (P(id)./D(id))/(G8*PSCALE)]';
function [PRIMIT,CONSER]=update(CELLS,GAMMAS,PRIMIT,CONSER,FLUXES,MESHPA)
%% prechod na dalsi casovou vrstvu
[~,~,~,~,~,~,~,~,G8]=GAMMAS{1:end};
[D,U,P]=PRIMIT{1:end};
CS=CONSER{1};
FI=FLUXES{1};
[DT,DX]=MESHPA{1:end};

DTODX=DT/DX;

for II=1:CELLS % konzervativni promenne
  I=II+2; % posun
  for K=1:3 % konzervativni promenne
    CS(K,I)=CS(K,I)+DTODX*(FI(K,I-1)-FI(K,I));
  end
end
for II=1:CELLS % konzervativni promenne
  I=II+2; % posun
  D(I)=CS(1,I);
  U(I)=CS(2,I)/D(I);
  P(I)=G8*(CS(3,I)-0.5*CS(2,I)*U(I));
end

PRIMIT={D,U,P};
CONSER={CS};

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [FLUXES,CONSER]=roe(CELLS,PRIMIT,SOUNDS,CONSER,GAMMAS)
%% Numericky tok podle Roea, viz Toro, 3-ti vydani, str. 345-376
[D,U,P]=PRIMIT{1:end}; % primitivni promenne
C=SOUNDS{1}; % rychlosti zvuku
CS=CONSER{1}; % konzervativni promenne
[GAMMA,~,~,~,~,~,~,~,G8]=GAMMAS{1:end}; % koeficienty 

FI=zeros(3,CELLS+4); % pseudodeklarace
FD=zeros(3,CELLS+4);

for II=0:CELLS+1
  I=II+2; % posun
  if (II<1)|| (II>CELLS) % konzervativni promenne
    CS(:,I)=[D(I); D(I)*U(I); 0.5*D(I)*U(I)*U(I)+P(I)/G8]; 
  end
  FD(:,I)=[CS(2,I); CS(2,I)*U(I)+P(I); U(I)*(CS(3,I)+P(I))];
end
%% reseni Riemannova problemu na hranici mezi bunkami I a I+1
for II=0:CELLS
  I=II+2; % posun
  DL=D(I);   UL=U(I);   PL=P(I);   CL=C(I);   EL=CS(3, I);  HL=(EL+PL)/DL;
  DR=D(I+1); UR=U(I+1); PR=P(I+1); CR=C(I+1); ER=CS(3,I+1); HR=(ER+PR)/DR;  
  UM=(sqrt(DL)*UL+sqrt(DR)*UR)/(sqrt(DL)+sqrt(DR)); % Roeova rychlost
  HM=(sqrt(DL)*HL+sqrt(DR)*HR)/(sqrt(DL)+sqrt(DR)); % Roeova entalpie
  CM2=G8*(HM-0.5*UM^2);
  assert(~(CM2<0),['***CHYBA***, metoda Roe, ',...
                   'zporn Roeova rychlost zvuku']);
  CM=sqrt(CM2); % Roeova rychlost zvuku
  TM=[1,UM-CM, HM-CM*UM;... % matice vlastnich vektoru
      1,UM,0.5*UM^2;...
      1,UM+CM,HM+CM*UM]';
  LAMBDA=[UM-CM,UM,UM+CM]; % vlastni cisla
  DELTA=CS(:,I+1)-CS(:,I); % W_R-W_L
  ALPHA=TM\DELTA; % koeficienty  
  if LAMBDA(2)>0 % pouzijeme levostrannou formuli, viz Toro, 11.129
    W1L=CS(:,I); % levy stav
    L1L=UL-CL; % \lambda_1^L: vlastni cislo leve
    W1R=W1L+ALPHA(1)*TM(:,1); % pravy stav
    D1R=W1R(1); 
    U1R=W1R(2)/D1R;
    P1R=G8*(W1R(3)-0.5*D1R*U1R^2);
    C1R2=GAMMA*P1R/D1R;
    assert(~(C1R2<0),['***CHYBA***, metoda Roe, ',...
                      'zporn Roeova rychlost zvuku']);
    C1R=sqrt(C1R2); % rychlost zvuku zprava
    L1R=U1R-C1R; % \lambda_1^R: vlastni cislo prave
     
    if (L1L<0) && (L1R>0) % LeVecque entropy fix: modifikace Lambda(1)  
      LAMBDA(1)=L1L*(L1R-LAMBDA(1))/(L1R-L1L); % viz Toro, 11.128   
    end
    FI(:,I)=FD(:,I)+min(0,LAMBDA(1))*ALPHA(1)*TM(:,1); % viz Toro, 11.129
  else % pouzijeme pravostrannou formuli, viz Toro, 11.133
    W3R=CS(:,I+1); % pravy stav
    L3R=UR+CR; % \lambda_3^R: vlastni cislo prave
    W3L=W3R-ALPHA(3)*TM(:,3); % levy stav
    D3L=W3L(1); 
    U3L=W3L(2)/D3L;
    P3L=G8*(W3L(3)-0.5*D3L*U3L^2);
    C3L2=GAMMA*P3L/D3L;
    assert(~(C3L2<0),['***CHYBA***, metoda Roe',...
                 'zporn Roeova rychlost zvuku']);
    C3L=sqrt(C3L2); % rychlost zvuku zleva
    L3L=U3L+C3L; % \lambda_3^L: vlastni cislo leve
    if (L3L<0) && (L3R>0) % % LeVecque entropy fix: modifikace Lambda(3)  
      LAMBDA(3)=L3R*(LAMBDA(3)-L3L)/(L3R-L3L); % viz Toro, 11.133   
    end
    FI(:,I)=FD(:,I+1)-max(0,LAMBDA(3))*ALPHA(3)*TM(:,3); % viz Toro, 11.134
  end % LAMBDA(2)
end

FLUXES={FI};
CONSER={CS};
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [FLUXES,CONSER]=hllc(CELLS,PRIMIT,SOUNDS,CONSER,GAMMAS)
%% Numericky tok HLLC, viz Toro, 3-ti vydani, str. 314-332
[D,U,P]=PRIMIT{1:end}; % primitivni promenne
C=SOUNDS{1}; % rychlosti zvuku
CS=CONSER{1}; % konzervativni promenne
[~,~,~,~,~,~,~,~,G8]=GAMMAS{1:end}; % koeficienty 

FI=zeros(3,CELLS+4); % pseudodeklarace
FD=zeros(3,CELLS+4);

for II=0:CELLS+1
  I=II+2; % posun
  if (II<1)|| (II>CELLS) % konzervativni promenne
    CS(:,I)=[D(I); D(I)*U(I); 0.5*D(I)*U(I)*U(I)+P(I)/G8]; 
  end
  FD(:,I)=[CS(2,I); CS(2,I)*U(I)+P(I); U(I)*(CS(3,I)+P(I))];
end
%% reseni Riemannova problemu na hranici mezi bunkami I a I+1
for II=0:CELLS
  I=II+2; % posun
  DL=D(I);   UL=U(I);   PL=P(I);   CL=C(I);
  DR=D(I+1); UR=U(I+1); PR=P(I+1); CR=C(I+1);
  STATES={DL,UL,PL,CL,DR,UR,PR,CR};
  [SL,SM,SR]=estime(GAMMAS,STATES); % odhad rychlosti vln
  if SL>=0 % nadzvukove proudeni doprava
    FI(:,I)=FD(:,I);
  elseif (SL<=0) && (SR>=0) % podzvukove proudeni, viz Toro, 10.38-10.39
    if SM>=0 % podzvukove proudeni doprava
      ENEL=CS(3,I)/DL;
      ENEL=ENEL+(SM-UL)*(SM+PL/(DL*(SL-UL)));
      CSL1=DL*(SL-UL)/(SL-SM);
      CSL=[CSL1;CSL1*SM;CSL1*ENEL];
      FI(:,I) = FD(:,I)+SL*(CSL-CS(:, I));
    else % podzvukove proudeni doleva
      ENER=CS(3,I+1)/DR;
      ENER=ENER+(SM-UR)*(SM+PR/(DR*(SR-UR)));
      CSR1=DR*(SR-UR)/(SR-SM);
      CSR=[CSR1;CSR1*SM;CSR1*ENER];
      FI(:,I)=FD(:,I+1)+SR*(CSR-CS(:,I+1));
    end
  elseif SR<=0 % supersonicke proudeni doleva
    FI(:,I)=FD(:,I+1);
  end  
end

FLUXES={FI};
CONSER={CS};
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [FLUXES,CONSER]=hll(CELLS,PRIMIT,SOUNDS,CONSER,GAMMAS)
%% Numericky tok HLL, viz Toro, 3-ti vydani, str. 320-322
[D,U,P]=PRIMIT{1:end}; % primitivni promenne
C=SOUNDS{1}; % rychlosti zvuku
CS=CONSER{1}; % konzervativni promenne
[~,~,~,~,~,~,~,~,G8]=GAMMAS{1:end}; % koeficienty 

FI=zeros(3,CELLS+4); % pseudodeklarace
FD=zeros(3,CELLS+4);

for II=0:CELLS+1
  I=II+2; % posun
  if (II<1)|| (II>CELLS) % konzervativni promenne
    CS(:,I)=[D(I); D(I)*U(I); 0.5*D(I)*U(I)*U(I)+P(I)/G8]; 
  end
  FD(:,I)=[CS(2,I); CS(2,I)*U(I)+P(I); U(I)*(CS(3,I)+P(I))];
end
%% reseni Riemannova problemu na hranici mezi bunkami I a I+1
for II=0:CELLS
  I=II+2; % posun
  DL=D(I);   UL=U(I);   PL=P(I);   CL=C(I);
  DR=D(I+1); UR=U(I+1); PR=P(I+1); CR=C(I+1);
  STATES={DL,UL,PL,CL,DR,UR,PR,CR};
  [SL,~,SR]=estime(GAMMAS,STATES); % odhad rychlosti vln
  if SL>=0 % nadzvukove proudeni doprava, viz Toro, 10.21
    FI(:,I)=FD(:,I);
  elseif (SL<=0) && (SR>=0) % podzvukove proudeni
    FI(:,I)=(SR*FD(:,I)-SL*FD(:,I+1)+SL*SR*(CS(:,I+1)-CS(:,I)))/(SR-SL);
  elseif SR<=0 % supersonicke proudeni doleva
    FI(:,I)=FD(:,I+1);
  end  
end

FLUXES={FI};
CONSER={CS};
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [FLUXES,CONSER]=rusanov(CELLS,PRIMIT,SOUNDS,CONSER,GAMMAS)
%% Numericky tok HLL, viz Toro, 3-ti vydani, str. 328-329
[D,U,P]=PRIMIT{1:end}; % primitivni promenne
C=SOUNDS{1}; % rychlosti zvuku
CS=CONSER{1}; % konzervativni promenne
[~,~,~,~,~,~,~,~,G8]=GAMMAS{1:end}; % koeficienty 

FI=zeros(3,CELLS+4); % pseudodeklarace
FD=zeros(3,CELLS+4);

for II=0:CELLS+1
  I=II+2; % posun
  if (II<1)|| (II>CELLS) % konzervativni promenne
    CS(:,I)=[D(I); D(I)*U(I); 0.5*D(I)*U(I)*U(I)+P(I)/G8]; 
  end
  FD(:,I)=[CS(2,I); CS(2,I)*U(I)+P(I); U(I)*(CS(3,I)+P(I))];
end
%% reseni Riemannova problemu na hranici mezi bunkami I a I+1
for II=0:CELLS
  I=II+2; % posun
  DL=D(I);   UL=U(I);   PL=P(I);   CL=C(I);
  DR=D(I+1); UR=U(I+1); PR=P(I+1); CR=C(I+1);
  STATES={DL,UL,PL,CL,DR,UR,PR,CR};
  [SL,~,SR]=estime(GAMMAS,STATES); % odhad rychlosti vln
  if SL>=0 % nadzvukove proudeni doprava, viz Toro, 10.21
    FI(:,I)=FD(:,I);
  elseif (SL<=0) && (SR>=0) % podzvukove proudeni, viz Toro, 10.55
    SPLUS=max(abs(SL),abs(SR));
    FI(:,I)=0.5*(FD(:,I)+FD(:,I+1))+0.5*SPLUS*(CS(:,I)-CS(:,I+1));
  elseif SR<=0 % supersonicke proudeni doleva
    FI(:,I)=FD(:,I+1);
  end  
end

FLUXES={FI};
CONSER={CS};
function [SL,SM,SR]=estime(GAMMAS,STATES)
%% odhad rychlosti vln pro solvery HLLC, HLL a Rusakov,
% algoritmus AN RS, Toro 305-306, obr. 9.4
[~,G1,G2,G3,G4,G5,G6,G7,~]=GAMMAS{1:end}; % koeficienty 
[DL,UL,PL,CL,DR,UR,PR,CR]=STATES{1:end};
 
CUP=0.25*(DL+DR)*(CL+CR); % \bar\varrho\bar a, viz Toro 10.67
PPV=0.5*(PL+PR)+0.5*(UL-UR)*CUP; % p_{pvrs}
PPV=max(0,PPV); % p_*
PMIN=min(PL,PR);
PMAX=max(PL,PR);
QMAX=PMAX/PMIN;
% ANRS, viz Toro, kapitola 9.5.2, obr. 9.4
QUSER=2;
if(QMAX<=QUSER) && ((PMIN<=PPV) && (PPV<=PMAX)) % Riemannuv solver PVRS
  PM=PPV; % p_*, Toro, 10.67
  UM=0.5*(UL+UR)+0.5*(PL-PR)/CUP; % u_*, viz Toro 9.20+9.21
else
  if PPV<PMIN  % two-Rarefaction Riemann solver
    PQ=(PL/PR)^G1;
    UM=(PQ*UL/CL+UR/CR+G4*(PQ-1))/(PQ/CL+1/CR); % u_*, viz Toro 9.35
    PTL=1+G7*(UL-UM)/CL;
    PTR=1+G7*(UM-UR)/CR;
    PM=0.5*(PL*PTL^G3+PR*PTR^G3); % p_*, viz Toro, 9.36
  else % two-Shock Riemann solver 
    GEL=sqrt((G5/DL)/(G6*PL+PPV));
    GER=sqrt((G5/DR)/(G6*PR+PPV));
    PM=(GEL*PL+GER*PR-(UR-UL))/(GEL+GER); %  p_*, viz Toro, 9.42
    UM=0.5*(UL+UR)+0.5*(GER*(PM-PR)-GEL*(PM-PL)); % u_*, viz Toro 9.43
  end
end
if PM<=PL % rychlost leve vlny, viz Toro 10.59-10.60
  SL=UL-CL;
else
  SL=UL-CL*sqrt(1+G2*(PM/PL-1.0));
end
SM=UM; % rychlost prostredni vlny
if PM<=PR % rychlost prave vlny, viz Toro 10.59-10.60
  SR=UR+CR;
else
  SR=UR+CR*sqrt(1+G2*(PM/PR-1));
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [FLUXES,CONSER]=vija(CELLS,PRIMIT,CONSER,GAMMAS)
%% Numericky tok Vijayasundaram, viz Feistauer,Felcman,Strakraba,str. 163
[D,U,P]=PRIMIT{1:end}; % primitivni promenne
CS=CONSER{1}; % konzervativni promenne
[GAMMA,~,~,~,~,~,~,~,G8]=GAMMAS{1:end}; % koeficienty 

FI=zeros(3,CELLS+4); % pseudodeklarace

for II=[0,CELLS+1]
  I=II+2; % posun
  CS(:,I)=[D(I); D(I)*U(I); 0.5*D(I)*U(I)*U(I)+P(I)/G8]; 
end
%% reseni Riemannova problemu na hranici mezi bunkami I a I+1
for II=0:CELLS
  I=II+2; % posun
  DL=D(I);   UL=U(I);   PL=P(I);
  DR=D(I+1); UR=U(I+1); PR=P(I+1);
  DA=0.5*(DL+DR); % prumerne hodnoty
  UA=0.5*(UL+UR);
  PA=0.5*(PL+PR);
  CA2=GAMMA*PA/DA;
  assert(~(CA2<0),['***CHYBA*** metoda : Vijayasundaram, ',...
                   'komplexn rychlost zvuku']);
  CA=sqrt(CA2);
  HA=CA*CA/(GAMMA-1)+0.5*UA*UA; 
  T=[1,UA-CA, HA-CA*UA;...
     1,UA,0.5*UA^2;...
     1,UA+CA,HA+UA*CA]';
  Lp=diag(max(0,[UA-CA,UA,UA+CA]));
  Lm=diag(min(0,[UA-CA,UA,UA+CA]));
  FI(:,I)=T*Lp*T^(-1)*CS(:,I)+T*Lm*T^(-1)*CS(:,I+1);
end

FLUXES={FI};
CONSER={CS};
function [FLUXES,CONSER]=stewa(CELLS,PRIMIT,SOUNDS,CONSER,GAMMAS)
%% Numericky tok Steger-Warming, viz Feistauer,Felcman,Strakraba,str. 163
[D,U,P]=PRIMIT{1:end}; % primitivni promenne
C=SOUNDS{1}; % rychlosti zvuku
CS=CONSER{1}; % konzervativni promenne
[~,~,~,~,~,~,~,~,G8]=GAMMAS{1:end}; % koeficienty 

FI=zeros(3,CELLS+4); % pseudodeklarace

for II=[0,CELLS+1]
  I=II+2; % posun
  CS(:,I)=[D(I); D(I)*U(I); 0.5*D(I)*U(I)*U(I)+P(I)/G8]; 
end
%% reseni Riemannova problemu na hranici mezi bunkami I a I+1
for II=0:CELLS
  I=II+2; % posun
  DL=D(I);   UL=U(I);   PL=P(I);   CL=C(I);   EL=CS(3, I);  HL=(EL+PL)/DL;
  DR=D(I+1); UR=U(I+1); PR=P(I+1); CR=C(I+1); ER=CS(3,I+1); HR=(ER+PR)/DR;   
  Tp=[1,UL-CL, HL-CL*UL;...
     1,UL,0.5*UL^2;...
     1,UL+CL,HL+UL*CL]';
  Tm=[1,UR-CR, HR-CR*UR;...
     1,UR,0.5*UR^2;...
     1,UR+CR,HR+UR*CR]';
  Lp=max(0,[UL-CL,UL,UL+CL]);
  Lm=min(0,[UR-CR,UR,UR+CR]);

%   LL=[UL-CL;UL;UL+CL];
%   LR=[UR-CR;UR;UR+CR];
%   tol=0.1; % toletance Choi entropy fix
%   for j=[1,3]
%     if ((LL(j)<0) && (0<LR(j)))
% %        Lp(j)=(Lp(j)+sqrt(Lp(j)^2+tol^2))/2; % Choi entropy fix
%         Lm(j)=(Lm(j)-sqrt(Lm(j)^2+tol^2))/2;
%     end
%   end
%   
   LP=diag(Lp);
   LM=diag(Lm);
  
  FI(:,I)=Tp*LP*Tp^(-1)*CS(:,I)+Tm*LM*Tm^(-1)*CS(:,I+1);
end

FLUXES={FI};
CONSER={CS};
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [FLUXES,CONSER]=vanleer1(CELLS,PRIMIT,SOUNDS,CONSER,GAMMAS)
%% Numericky tok Van Leer, viz Feistauer,Felcman,Strakraba,str. 164
[D,U,P]=PRIMIT{1:end}; % primitivni promenne
C=SOUNDS{1}; % rychlosti zvuku
CS=CONSER{1}; % konzervativni promenne
[GAMMA,~,~,~,~,~,~,~,G8]=GAMMAS{1:end}; % koeficienty 

FI=zeros(3,CELLS+4); % pseudodeklarace
FD=zeros(3,CELLS+4);

for II=0:CELLS+1
  I=II+2; % posun
  if (II<1)|| (II>CELLS) % konzervativni promenne
    CS(:,I)=[D(I); D(I)*U(I); 0.5*D(I)*U(I)*U(I)+P(I)/G8]; 
  end
  FD(:,I)=[CS(2,I); CS(2,I)*U(I)+P(I); U(I)*(CS(3,I)+P(I))];
end
%% reseni Riemannova problemu na hranici mezi bunkami I a I+1
for II=0:CELLS
  I=II+2; % posun
  DL=D(I);   UL=U(I);   PL=P(I);   CL=C(I);
  DR=D(I+1); UR=U(I+1); PR=P(I+1); CR=C(I+1);
  
  DA=0.5*(DL+DR); % prumerne hodnoty
  UA=0.5*(UL+UR);
  PA=0.5*(PL+PR);
  CA2=GAMMA*PA/DA;
  assert(~(CA2<0),['***CHYBA*** metoda : Van Leer 1, ',...
                   'komplexn rychlost zvuku']);
  CA=sqrt(CA2);
  HA=CA*CA/(GAMMA-1)+0.5*UA*UA;
  
  T=[1,UA-CA, HA-CA*UA;...
     1,UA,0.5*UA^2;...
     1,UA+CA,HA+UA*CA]';
  LA=abs([UA-CA,UA,UA+CA]);
  
  LL=[UL-CL;UL;UL+CL];
  LR=[UR-CR;UR;UR+CR];
  tol=0.5; % toletance pro Hartenuv entropy fix
  for j=[1,3]
    if ((LL(j)<0) && (0<LR(j))) && (LA(j)<tol)
      LA(j)=(LA(j)^2+tol^2)/(2*tol); % Hartenuv entropy fix
    end
  end
  
  Da=diag(LA);  
  FI(:,I)=0.5*(FD(:,I)+FD(:,I+1)-T*Da*T^(-1)*(CS(:,I+1)-CS(:,I)));
end

FLUXES={FI};
CONSER={CS};
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [FLUXES,CONSER]=vanleer2(CELLS,PRIMIT,SOUNDS,CONSER,GAMMAS)
%% Numericky tok Van Leer, viz Toro, str. 277-278
[D,U,P]=PRIMIT{1:end}; % primitivni promenne
C=SOUNDS{1}; % rychlosti zvuku
CS=CONSER{1}; % konzervativni promenne
[GAMMA,~,~,~,~,~,~,~,G8]=GAMMAS{1:end}; % koeficienty 

FI=zeros(3,CELLS+4); % pseudodeklarace

for II=[0,CELLS+1]
  I=II+2; % posun
  CS(:,I)=[D(I); D(I)*U(I); 0.5*D(I)*U(I)*U(I)+P(I)/G8]; 
end
%% reseni Riemannova problemu na hranici mezi bunkami I a I+1
for II=0:CELLS
  I=II+2; % posun
  DL=D(I);   UL=U(I);   CL=C(I);
  DR=D(I+1); UR=U(I+1); CR=C(I+1); 
  
  ML=UL/CL; % Machovo cislo zleva
  QL=0.25*DL*CL*(1+ML)^2;
  ZL=[QL;... % viz Toro, 8.76
     QL*(2*CL/GAMMA*(0.5*(GAMMA-1)*ML+1));...
     QL*(2*CL^2/(GAMMA^2-1)*(0.5*(GAMMA-1)*ML+1)^2)];
  MR=UR/CR; % Machovo cislo zprava
  QR=-0.25*DR*CR*(1-MR)^2;
  ZR=[QR;... % viz Toro, 8.77
      QR*(2*CR/GAMMA*(0.5*(GAMMA-1)*MR-1));
	  QR*(2*CR^2/(GAMMA^2-1)*(0.5*(GAMMA-1)*MR-1)^2)];
  FI(:,I)=ZL+ZR;         
end

FLUXES={FI};
CONSER={CS};
