function [Xout,Xsol]=e1r(Xin,tisk)
%% reseni Riemannova problemu
% prepis programu uvedeneho v knize
% E.F.Toro: Riemann Solvers and Numerical Methods for Fluid Dynamics,
% Springer-Verlag, Berlin, 2009 
%
% vstupni data:
%
% Xin  ... vstupni data, strukura obsahujici
%
%   DOMLEN ... delka intervalu
%   DIAPH1 ... poloha prepazky
%   CELLS  ... pocet bunek
%   GAMMA  ... Poissonova adiabaticka konstanta
%   TIMEOU ... doba reseni
%   DL     ... pocatecni hustota vlevo
%   UL     ... oocatecni rychlost vlevo
%   PL     ... pocatecni tlak vlevo
%   DR     ... pocatecni hustota vpravo
%   UR     ... pocatecni rychlost vprvo
%   PR     ... pocatecni tlak vpravo
%   PSCALE ... normalizacni konstanta
%
% tisk   ...  = true  ==> pomocne tisky solveru
%             = false ==> bez pomocnych tisku solveru
%
% vystupni udaje:
%
% Xout ... pole typu (5,CELLS), jejiz jednotlive radky obsahuji: 
%          
% Xout(1,:) ... souradnice stredu bunek
% Xout(2,:) ... hustoty ve stredech bunek
% Xout(3,:) ... rychlosti ve stredech bunek
% Xout(4,:) ... tlaky ve stredech bunek
% Xout(5,:) ... merne vnitrni energie ve stredech bunek
%%
[DOMLEN,DIAPH1,CELLS,GAMMA,TIMEOU,DL,UL,PL,DR,UR,PR,PSCALE]=Xin{1:end};
% konstanty gamma
G1=(GAMMA-1)/(2*GAMMA);
G2=(GAMMA+1)/(2*GAMMA);
G3=2*GAMMA/(GAMMA-1);
G4=2/(GAMMA-1);
G5=2/(GAMMA+1);
G6=(GAMMA-1)/(GAMMA+1);
G7=(GAMMA-1)/2;
G8=GAMMA-1;
% rychlosti zvuku
CL = sqrt(GAMMA*PL/DL);
CR = sqrt(GAMMA*PR/DR);
% je tlak kladny?
if G4*(CL+CR)<=UR-UL % jstliza pocatecni data generuji valuum ==> stop      
  fprintf(['***Vacuum is generated by data***\n',...
           '***Program stopped***\n']);
  quit;
end
GAMMAS={GAMMA,G1,G2,G3,G4,G5,G6,G7,G8}; % struktura GAMMAS
STATES={DL,UL,PL,CL,DR,UR,PR,CR}; % struktura STATES
[PM,UM]=starpu(PSCALE,GAMMAS,STATES,tisk); % tlak a rychlost ve star-oblasti
 DX = DOMLEN/CELLS; % delka bunky
% vypocet eseni v case TIME
Xout=zeros(5,CELLS); % pseudodeklarace
for I=1:CELLS
  XPOS=(I-0.5)*DX;
  S=(XPOS-DIAPH1)/TIMEOU;
%   reseni v bode (X,T)=(XPOS-DIAPH1,TIMEOU)
  [DS,US,PS]=sample(S,PM,UM,GAMMAS,STATES);
  Xout(:,I)=[XPOS,DS,US,PS/PSCALE,PS/DS/G8/PSCALE]';
end 
if PM<=PL % leva vlna zredeni
  DML=DL*(PM/PL)^(1/GAMMA);
  SHL=UL-CL; % celo vlevo
  CML=CL*(PM/PL)^G1; 
  STL=UM-CML; % tyl vlevo, SHL < STL
else % leva razova vlna
  PML=PM/PL;
  DML=DL*(PML+G6)/(PML*G6+1);
  SHL=UL-CL*sqrt(G2*PML+G1); % rychlost leve razove vlny
  STL=SHL;
end  
if PM>PR % prava razova vlna
  PMR=PM/PR;
  DMR=DR*(PMR+G6)/(PMR*G6+1);
  SHR=UR+CR*sqrt(G2*PMR+G1); % rychlost prave razove vlny
  STR=SHR;
else % prava vlna zredeni
  DMR=DR*(PM/PR)^(1/GAMMA);
  SHR=UR+CR; % prave celo
  CMR=CR*(PM/PR)^G1;
  STR=UM+CMR; % pravy tyl, STR < SHR
end
Xsol=[DML,DMR,UM,PM,SHL,STL,STR,SHR];
function [P,U]=starpu(PSCALE,GAMMAS,STATES,tisk)
%% vypocet tlaku a rychlosti ve star-oblasti
% vstupni udaje: 
%
% PSCALE ... meritko
% GAMMAS ... konstanty
% STATES ... stavove veliciny
% tisk   ...  = true  ==> pomocne tisky solveru
%             = false ==> bez pomocnych tisku solveru
%
% vystupni udaje:
%
% P ... tlak ve star-oblasti
% U ... rychlost ve star-oblasti
%
%%
% if tisk
%   fprintf('\nsolver:\n\n');
% end
[DL,UL,PL,CL,DR,UR,PR,CR]=STATES{1:end};
TOLPRE=1e-6; % predepsana tolerance
NRITER=20; % maximalni pocet iteraci Newtonovy metody

PSTART=guessp(GAMMAS,STATES); % odhad tlaku
POLD=PSTART;
UDIFF=UR-UL;
if tisk
  fprintf(['-----------------------------------------\n',...
           '  iterace                    zmena  \n',...
           '-----------------------------------------\n']);
end
failed=true;
for I=1:NRITER % reseni nelinearni rovnice Newtonovou metodou
  [FL,FLD]=prefun(POLD,DL,PL,CL,GAMMAS);
  [FR,FRD]=prefun(POLD,DR,PR,CR,GAMMAS);
  P=POLD-(FL+FR+UDIFF)/(FLD+FRD);
  CHANGE=2*abs((P-POLD)/(P+POLD));
  if tisk
    fprintf('     %4d              %12.7f\n',I,CHANGE);
  end
  if CHANGE<=TOLPRE, failed=false; break; end;
  if P<0, P=TOLPRE; end;
  POLD=P;
end
if failed
  fprintf('Newtonova metoda nekonverguje\n');
  quit;
end
%fprintf('----------------------------------------\n');
U=0.5*(UL+UR+FR-FL); % rychlost ve star-oblasti
if tisk
  fprintf(['-----------------------------------------\n',...
           'tlak p*     = %10.4f\n'...
           'rychlost u* = %10.4f\n',...
           '-----------------------------------------\n'],P/PSCALE,U);  
end
function PM=guessp(GAMMAS,STATES)
%% vypocet odhadu tlaku ve star-oblasti
%
% vstupni udaje: 
%
% GAMMAS ... konstanty
% STATES ... stavove veliciny
%
% vystupni udaje:
%
% PM ... odhad hodtoty tlaku ve star-oblasti
%%
[DL,UL,PL,CL,DR,UR,PR,CR]=STATES{1:end};
[~,G1,~,G3,G4,G5,G6,G7,~]=GAMMAS{1:end};
QUSER = 2.0;
CUP=0.25*(DL+DR)*(CL+CR);
PPV=0.5*(PL+PR)+0.5*(UL-UR)*CUP;
PPV=max(0,PPV);
PMIN=min(PL,PR);
PMAX=max(PL,PR);
QMAX=PMAX/PMIN;
if (QMAX<=QUSER)&&((PMIN<=PPV)&&(PPV<=PMAX))
  PM=PPV;
else
  if PPV<PMIN 
    PQ=(PL/PR)^G1;
    UM=(PQ*UL/CL+UR/CR+G4*(PQ-1))/(PQ/CL+1/CR);
    PTL=1+G7*(UL-UM)/CL;
    PTR=1+G7*(UM-UR)/CR;
    PM=0.5*(PL*PTL^G3+PR*PTR^G3);
  else
    GEL=sqrt((G5/DL)/(G6*PL+PPV));
    GER=sqrt((G5/DR)/(G6*PR+PPV));
    PM=(GEL*PL+GER*PR-(UR-UL))/(GEL+GER);
  end
end
function [F,FD]=prefun(P,DK,PK,CK,GAMMAS)
%% vyhodnoceni funkci FL a FR a jejich prvnich derivaci
%
% vstupni udaje:
%
% P      ... tlak
% DK     ... hustota
% PK     ... tlak
% CK     ... rychlost zvuku
% GAMMAS ... konstanty
%
% vystupni udaje:
%
% F  ... hodnota funkce tlaku
% FD ... derivace funkcetlaku
%%
[~,G1,G2,~,G4,G5,G6,~,~]=GAMMAS{1:end};
if P<=PK % vlna zredeni
  PRATIO=P/PK;
  F=G4*CK*(PRATIO^G1-1);
  FD=(1/(DK*CK))*PRATIO^(-G2);
else % razova vlna
  AK=G5/DK;
  BK=G6*PK;
  QRT=sqrt(AK/(BK+P));
  F=(P-PK)*QRT;
  FD=(1-0.5*(P-PK)/(BK+P))*QRT;
end;
function [D,U,P]=sample(S,PM,UM,GAMMAS,STATES)
%% vypocet hustoty, rychlosti a tlaku v bodu S=X/T
%
% vstupni udaje:
%
% S ... = X/T, kde X je x-ova souradnice a T je cas
% PM ... tlak ve star-zone
% UM ... rychlost ve star-zone
% GAMMAS ... konstanty
% STATES ... stavove veliciny
%
% vystupni udaje:
%
% D ... hustota
% U ... rychlost
% P ... tlak
%%
[DL,UL,PL,CL,DR,UR,PR,CR]=STATES{1:end};
[GAMMA,G1,G2,G3,G4,G5,G6,G7,~]=GAMMAS{1:end};
if S<=UM % S je vlevo od kontaktni nespojitosti
  if PM<=PL % leva vlna zredeni
     SHL=UL-CL;
     if S<=SHL % navlevo od leve vlny zredeni
       D=DL;
       U=UL;
       P=PL;
     else % napravo od leve vlny zredeni
       CML=CL*(PM/PL)^G1;
       STL=UM-CML;
       if S>STL 
         D=DL*(PM/PL)^(1.0/GAMMA);
         U=UM;
         P=PM;
       else % uvnitr leve vlny zredeni
         U=G5*(CL + G7*UL + S);
         C=G5*(CL + G7*(UL - S));
         D=DL*(C/CL)^G4;
         P=PL*(C/CL)^G3;
       end
     end
  else % leva razova vlna
    PML=PM/PL;
    SL=UL-CL*sqrt(G2*PML+G1);
    if S<=SL % nalevo od leve razove vlny 
      D=DL;
      U=UL;
      P=PL;
    else % napravo od leve razove vlny
      D=DL*(PML+G6)/(PML*G6+1);
      U=UM;
      P=PM;
    end
  end
else % napravo od kontaktni nespojitosti
  if PM>PR % prava razova vlna
    PMR=PM/PR;
    SR=UR+CR*sqrt(G2*PMR+G1);
    if S>=SR % napravo od prave razove vlny
      D=DR;
      U=UR;
      P=PR;
    else % nalevo od prave razove vlny
      D=DR*(PMR+G6)/(PMR*G6+1);
      U=UM;
      P=PM;
    end;
  else % prava vlna zredeni
    SHR=UR+CR;
    if S>=SHR % napravo od prave vlny zredeni
      D=DR;
      U=UR;
      P=PR;
    else
      CMR=CR*(PM/PR)^G1;
      STR=UM+CMR;
      if S<=STR % nalevo od prave vlny zredeni
        D=DR*(PM/PR)^(1/GAMMA);
        U=UM;
        P=PM;
      else % uvnitr prave vlny zredeni
        U=G5*(-CR+G7*UR+S);
        C=G5*(CR-G7*(UR-S));
        D=DR*(C/CR)^G4;
        P=PR*(C/CR)^G3;
      end;
    end
  end
end